@@ -64,7 +64,7 @@ namespace PyXRootD
     pystatus = ConvertType<XrdCl::XRootDStatus>( &status );
     PyObject *o = ( callback && callback != Py_None ) ?
             Py_BuildValue( "O", pystatus ) :
-            Py_BuildValue( "OO", pystatus, Py_BuildValue( "" ) );
+            Py_BuildValue( "ON", pystatus, Py_BuildValue( "" ) );
     Py_DECREF( pystatus );
     return o;
   }
@@ -95,7 +95,7 @@ namespace PyXRootD
     pystatus = ConvertType<XrdCl::XRootDStatus>( &status );
     PyObject *o = ( callback && callback != Py_None ) ?
             Py_BuildValue( "O", pystatus ) :
-            Py_BuildValue( "OO", pystatus, Py_BuildValue( "" ) );
+            Py_BuildValue( "ON", pystatus, Py_BuildValue( "" ) );
     Py_DECREF( pystatus );
     return o;
   }
@@ -254,12 +254,12 @@ namespace PyXRootD
     if ( size < chunksize ) chunksize = size;
 
     uint64_t off_end = offset + size;
-    XrdCl::Buffer* chunk = new XrdCl::Buffer();
-    XrdCl::Buffer* line = new XrdCl::Buffer();
+    std::unique_ptr<XrdCl::Buffer> chunk;
+    std::unique_ptr<XrdCl::Buffer> line = std::make_unique<XrdCl::Buffer>();
 
     while ( offset < off_end )
     {
-      chunk = self->ReadChunk( self, offset, chunksize );
+      chunk.reset( self->ReadChunk( self, offset, chunksize ) );
       offset += chunk->GetSize();
 
       // Reached end of file
@@ -300,8 +300,6 @@ namespace PyXRootD
     else
       pyline = PyUnicode_FromString( "" );
 
-    delete line;
-    delete chunk;
     return pyline;
   }
 
@@ -350,6 +348,7 @@ namespace PyXRootD
         break;
 
       PyList_Append( lines, line );
+      Py_DECREF( line );
     }
 
     return lines;
@@ -406,7 +405,7 @@ namespace PyXRootD
 
     if ( PyType_Ready( &ChunkIteratorType ) < 0 ) return NULL;
 
-    args = Py_BuildValue( "OOO", self, Py_BuildValue("k", offset),
+    args = Py_BuildValue( "ONN", self, Py_BuildValue("k", offset),
                                        Py_BuildValue("I", chunksize) );
     iterator = (ChunkIterator*)
                PyObject_CallObject( (PyObject *) &ChunkIteratorType, args );
@@ -472,7 +471,7 @@ namespace PyXRootD
     pystatus = ConvertType<XrdCl::XRootDStatus>( &status );
     PyObject *o = ( callback && callback != Py_None ) ?
             Py_BuildValue( "O", pystatus ) :
-            Py_BuildValue( "OO", pystatus, Py_BuildValue( "" ) );
+            Py_BuildValue( "ON", pystatus, Py_BuildValue( "" ) );
     Py_DECREF( pystatus );
     return o;
   }
@@ -504,7 +503,7 @@ namespace PyXRootD
     pystatus = ConvertType<XrdCl::XRootDStatus>( &status );
     PyObject *o = ( callback && callback != Py_None ) ?
             Py_BuildValue( "O", pystatus ) :
-            Py_BuildValue( "OO", pystatus, Py_BuildValue( "" ) );
+            Py_BuildValue( "ON", pystatus, Py_BuildValue( "" ) );
     Py_DECREF( pystatus );
     return o;
   }
@@ -551,7 +550,7 @@ namespace PyXRootD
     pystatus = ConvertType<XrdCl::XRootDStatus>( &status );
     PyObject *o = ( callback && callback != Py_None ) ?
             Py_BuildValue( "O", pystatus ) :
-            Py_BuildValue( "OO", pystatus, Py_BuildValue( "" ) );
+            Py_BuildValue( "ON", pystatus, Py_BuildValue( "" ) );
     Py_DECREF( pystatus );
     return o;
   }
@@ -587,6 +586,17 @@ namespace PyXRootD
       return NULL;
     }
 
+    struct chunkGuard {
+      chunkGuard(XrdCl::ChunkList &c) : c_(&c) { }
+      ~chunkGuard() {
+        if (c_)
+          std::for_each(c_->begin(), c_->end(), [](XrdCl::ChunkInfo &ci) { delete[] (char*)ci.buffer; });
+      }
+      void disarm() { c_ = nullptr; }
+
+       XrdCl::ChunkList *c_;
+    } cg(chunks);
+
     for ( int i = 0; i < PyList_Size( pychunks ); ++i ) {
       PyObject *chunk = PyList_GetItem( pychunks, i );
 
@@ -616,11 +626,13 @@ namespace PyXRootD
       XrdCl::ResponseHandler *handler
           = GetHandler<XrdCl::VectorReadInfo>( callback );
       if ( !handler ) return NULL;
+      cg.disarm(); // handler will call ConvertType and free chunk buffers
       async( status = self->file->VectorRead( chunks, 0, handler, timeout ) );
     }
     else {
       XrdCl::VectorReadInfo *info = 0;
       async( status = self->file->VectorRead( chunks, 0, info, timeout ) );
+      cg.disarm(); // ConvertType will free chunk buffers
       pyresponse = ConvertType<XrdCl::VectorReadInfo>( info );
       delete info;
     }
