@@ -27,8 +27,8 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
 
-#include <stdio.h>
-#include <string.h>
+#include <cstdio>
+#include <cstring>
 
 #include "XrdAcc/XrdAccEntity.hh"
 #include "XrdOuc/XrdOucTokenizer.hh"
