@@ -30,10 +30,13 @@
 
 #include <cerrno>
 #include <poll.h>
+#include <sys/uio.h>
 
 #include "XrdNet/XrdNet.hh"
 #include "XrdNet/XrdNetMsg.hh"
 #include "XrdNet/XrdNetOpts.hh"
+#include "XrdNet/XrdNetPeer.hh"
+#include "XrdNet/XrdNetRefresh.hh"
 #include "XrdSys/XrdSysError.hh"
 #include "XrdSys/XrdSysPlatform.hh"
 
@@ -41,55 +44,123 @@
 /*                           C o n s t r u c t o r                            */
 /******************************************************************************/
   
-XrdNetMsg::XrdNetMsg(XrdSysError *erp, const char *dest, bool *aOK)
+XrdNetMsg::XrdNetMsg(XrdSysError *erp, const char *dest, bool *aOK, bool refr) 
+                    : eDest(erp)
 {
    XrdNet myNet(erp);
-   bool   aok = true;
+   bool aok = true;
 
-   eDest = erp; FD = -1; destOK = 0;
-   if (dest)
-      {const char *eText = dfltDest.Set(dest);
-       if (!eText) destOK = 1;
-          else {eDest->Emsg("Msg", "Default", dest, "is unreachable");
-                aok = false;
-               }
+// Handle the case where no dest was specified. In this case we will always
+// need the caller to specify a destination.
+//
+   if (!dest)
+      {if ((FD = myNet.Relay(dest)) < 0)
+         {eDest->Emsg("NetMsg", "Unable to create UDP msg socket.");
+          aok = false;
+         }
+       if (aOK) *aOK = aok;
+       return;
       }
 
-    if ((FD = myNet.Relay(dest)) < 0)
-       {eDest->Emsg("Msg", "Unable to create UDP msg socket.");
-        aok = false;
-       }
+// Hande the common case where a dest is specified. We first make
+// sure the dest is valid, eventhough that will occur again, so we van
+// generate a resonable error message.
+//      
+   XrdNetAddr specDest;
+   const char *eText = specDest.Set(dest);
+   if (eText)
+      {eDest->Emsg("NetMsg", "Default", dest, "is unreachable");
+       if (aOK) *aOK = false;
+       return;
+      }
 
-    if (aOK) *aOK = aok;
+// Obtain a file description for this socket and set the endpoint address
+//
+   XrdNetPeer myPeer;
+
+   if (!myNet.Relay(myPeer, dest, XRDNET_SENDONLY))
+      {eDest->Emsg("NetMsg", "Unable to create UDP msg socket.");
+       if (aOK) *aOK = false;
+       return;
+      }
+
+// Save the relevant information
+//
+   dfltDest = strdup(myPeer.InetName ? myPeer.InetName : "Unknow!");
+   FD       = myPeer.fd;
+   destOK   = true;
+
+// If address refresh wanted, register this socket for refresh. This should
+// never fail and if it does we return non-success.
+//
+   if (refr && !XrdNetRefresh::Register(myPeer)) aok = false;
+
+// All done
+//
+   if (aOK) *aOK = aok;
 }
 
+/******************************************************************************/
+/*                            D e s t r u c t o r                             */
+/******************************************************************************/
+
+XrdNetMsg::~XrdNetMsg()
+{
+// If we are registered,deregister
+//
+   if (isRefr) XrdNetRefresh::UnRegister(FD);
+
+// Close the socket
+//
+   if (close(FD) < 0)
+      eDest->Emsg("NetMsg", errno, "close socket for", dfltDest);
+
+// Free the poiinter to the default dest
+//
+   free(dfltDest);
+}
+  
 /******************************************************************************/
 /*                                  S e n d                                   */
 /******************************************************************************/
   
 int XrdNetMsg::Send(const char *Buff, int Blen, const char *dest, int tmo)
 {
-   XrdNetAddr *theDest;
    int retc;
 
+// Get the buffer length of not specified
+//
    if (!Blen && !(Blen = strlen(Buff))) return  0;
 
+// Handle the case where we are sendingto he dest setup at construction. This
+// is the most common case.
+//
    if (!dest)
        {if (!destOK)
-           {eDest->Emsg("Msg", "Destination not specified."); return -1;}
-        theDest = &dfltDest;
+           {eDest->Emsg("NetMsg", "Destination not specified."); return -1;}
+
+        if (tmo >= 0 && !OK2Send(tmo, dfltDest)) return 1;
+
+        do {retc = send(FD, (Sokdata_t)Buff, Blen, 0);
+           } while (retc < 0 && errno == EINTR);
+
+        return (retc < 0 ? retErr(errno, dfltDest) : 0);
        }
-      else if (specDest.Set(dest))
-              {eDest->Emsg("Msg", dest, "is unreachable");    return -1;}
-              else theDest = &specDest;
+
+// Caller want to send to a specific destination other than the default
+//
+   XrdNetAddr specDest;
+
+   if (specDest.Set(dest))
+      {eDest->Emsg("NetMsg", dest, "is unreachable"); return -1;}
 
    if (tmo >= 0 && !OK2Send(tmo, dest)) return 1;
 
    do {retc = sendto(FD, (Sokdata_t)Buff, Blen, 0, 
-                     theDest->SockAddr(), theDest->SockSize());}
+                     specDest.SockAddr(), specDest.SockSize());}
        while (retc < 0 && errno == EINTR);
 
-   return (retc < 0 ? retErr(errno, theDest) : 0);
+   return (retc < 0 ? retErr(errno, specDest) : 0);
 }
 
 /******************************************************************************/
@@ -103,15 +174,15 @@ int XrdNetMsg::Send(const char *dest, const XrdNetSockAddr &netSA,
 
    if (netSA.Addr.sa_family == AF_INET) aSize = sizeof(netSA.v4);
       else if (netSA.Addr.sa_family == AF_INET6) aSize = sizeof(netSA.v6);
-              else return -1;
+              else return retErr(EAFNOSUPPORT, (dest ? dest : "Unknown!"));
 
    if (tmo >= 0 && !OK2Send(tmo, dest)) return 1;
 
    do {retc = sendto(FD, (Sokdata_t)Buff, Blen, 0, &netSA.Addr, aSize);}
        while (retc < 0 && errno == EINTR);
 
-   if (retc >= 0) return 1;
-   return (EWOULDBLOCK == errno || EAGAIN == errno ? 1 : -1);
+   if (retc >= 0) return 0;
+   return retErr(errno, (dest ? dest : "Unknown!"));
 }
 
 /******************************************************************************/
@@ -119,19 +190,45 @@ int XrdNetMsg::Send(const char *dest, const XrdNetSockAddr &netSA,
 int XrdNetMsg::Send(const struct iovec iov[], int iovcnt, 
                     const char  *dest,        int tmo)
 {
-   char buff[4096], *bp = buff;
-   int i, dsz = sizeof(buff);
 
+// Handle the common case of sendingto the cobbected address
+//
+   if (!dest)
+      {if (!destOK)
+          {eDest->Emsg("NetMsg", "Destination not specified."); return -1;}
+       if (tmo >= 0 && !OK2Send(tmo, dfltDest)) return 1;
+       if (writev(FD, iov, iovcnt) >= 0) return 0;
+       return retErr(errno, dfltDest);
+      }
+
+// Caller want to send to a specific destination other than the default
+//
+   XrdNetAddr specDest;
+   int retc;
+
+   if (specDest.Set(dest))
+      {eDest->Emsg("NetMsg", dest, "is unreachable"); return -1;}
+
+// Create the message via the msghdr
+//
+   struct msghdr mHdr{};
+
+   mHdr.msg_name = (void*)specDest.SockAddr();
+   mHdr.msg_namelen = specDest.SockSize();
+   mHdr.msg_iov = const_cast<struct iovec*>(iov);
+   mHdr.msg_iovlen = iovcnt;
+
+// Handle timeout if need be  
+//
    if (tmo >= 0 && !OK2Send(tmo, dest)) return 1;
 
-   for (i = 0; i < iovcnt; i++)
-       {dsz -= iov[i].iov_len;
-        if (dsz < 0) return retErr(EMSGSIZE, dest);
-        memcpy((void *)bp,(const void *)iov[i].iov_base,iov[i].iov_len);
-        bp += iov[i].iov_len;
-       }
+// Send the message
+//
+   do {retc = sendmsg(FD, &mHdr, 0);} while (retc < 0 && errno == EINTR);
 
-   return Send(buff, (int)(bp-buff), dest, -1);
+// All done
+//
+   return (retc < 0 ? retErr(errno, specDest) : 0);
 }
   
 /******************************************************************************/
@@ -149,9 +246,9 @@ int XrdNetMsg::OK2Send(int timeout, const char *dest)
    do {retc = poll(&polltab, 1, timeout);} while(retc < 0 && errno == EINTR);
 
    if (retc == 0 || !(polltab.revents & (POLLOUT | POLLWRNORM)))
-      eDest->Emsg("Msg", "UDP link to", dest, "is blocked.");
+      eDest->Emsg("NetMsg", "UDP link to", dest, "is blocked.");
       else if (retc < 0)
-              eDest->Emsg("Msg",errno,"poll", dest);
+              eDest->Emsg("NetMsg",errno,"poll", dest);
               else return 1;
    return 0;
 }
@@ -164,14 +261,14 @@ int XrdNetMsg::retErr(int ecode, const char *theDest)
 {
    if (!theDest)
       {if (!destOK)
-          {eDest->Emsg("Msg", "Destination not specified."); return -1;}
-       theDest = dfltDest.Name("unknown");
+          {eDest->Emsg("NetMsg", "Destination not specified."); return -1;}
+       theDest = dfltDest;
       }
-   eDest->Emsg("Msg", ecode, "send to", theDest);
+   eDest->Emsg("NetMsg", ecode, "send to", theDest);
    return (EWOULDBLOCK == ecode || EAGAIN == ecode ? 1 : -1);
 }
   
-int XrdNetMsg::retErr(int ecode, XrdNetAddr *theDest)
+int XrdNetMsg::retErr(int ecode, XrdNetAddr& theDest)
 {
-   return retErr(ecode, theDest->Name("unknown"));
+   return retErr(ecode, theDest.Name("unknown"));
 }
