@@ -28,11 +28,11 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
 
-#include <errno.h>
+#include <cerrno>
 #include <fcntl.h>
-#include <string.h>
-#include <time.h>
-#include <stdio.h>
+#include <cstring>
+#include <ctime>
+#include <cstdio>
 #include <unistd.h>
 #include <utime.h>
 #include <sys/mman.h>
@@ -50,7 +50,7 @@
   
 extern XrdSysError OssEroute;
 
-extern XrdOucTrace OssTrace;
+extern XrdSysTrace OssTrace;
 
 /******************************************************************************/
 /* Public:                          C o p y                                   */
