@@ -78,7 +78,7 @@ int posix_fallocate(int fd, off_t offset, off_t len)
     off_t c_test;
     int ret;
     if (!__builtin_saddll_overflow(offset, len, &c_test)) {
-        fstore_t store = {F_ALLOCATECONTIG, F_PEOFPOSMODE, 0, offset + len};
+        fstore_t store = {F_ALLOCATECONTIG, F_PEOFPOSMODE, 0, offset + len, 0};
         // Try to get a continous chunk of disk space
         ret = fcntl(fd, F_PREALLOCATE, &store);
         if (ret < 0) {
