@@ -194,7 +194,8 @@ void FileCopyTest::UploadTestFunc()
   //----------------------------------------------------------------------------
   FileSystem  fs( url );
   LocationInfo *locations = nullptr;
-  EXPECT_XRDST_OK( fs.DeepLocate( remoteFile, OpenFlags::Refresh, locations ) );
+  OpenFlags::Flags flags = OpenFlags::PrefName | OpenFlags::Refresh;
+  EXPECT_XRDST_OK( fs.DeepLocate( remoteFile, flags, locations ) );
   ASSERT_TRUE( locations );
   EXPECT_NE( locations->GetSize(), 0 );
   FileSystem fs1( locations->Begin()->GetAddress() );
